//dramfs.c
//DRAM "filesystem" to hold graphics blobs
//Bryan Topp <betopp@betopp.com> 2015

#include <string.h>
#include <stdio.h>

#include "dram.h"
#include "dramfs.h"

static unsigned int dramfs_valid = 0;
static unsigned int dramfs_num_files = 0;

void dramfs_init()
{	
	printf("DRAMFS init...\n");
	
	if(memcmp(DRAMFS_LOC, "betdramfs_00", 12) != 0)
	{
		fprintf(stderr, "Failed to read magic.\n");
		return;
	}
	
	dramfs_num_files = *(unsigned int*)(DRAMFS_LOC+12);
	printf("DRAMFS blob contains %u files.\n", dramfs_num_files);
	
	if(dramfs_num_files == 0)
		fprintf(stderr, "Warning: 0 files in DRAMFS!\n");
	
	printf("DRAMFS load successful.\n");
	dramfs_valid = 1;
}

void dramfs_clobbered()
{
	dramfs_valid = 0;
	dramfs_num_files = 0;
}

const char *dramfs_getname(unsigned int file_index)
{
	if(file_index >= dramfs_num_files || !dramfs_valid)
		return "";
	
	//Look up the name pointer in the directory
	unsigned int *directory = (unsigned int*)(DRAMFS_LOC + 16);
	unsigned int offset = directory[file_index*3];
	
	return (const char*)(DRAMFS_LOC+offset);
}

const void *dramfs_getptr(unsigned int file_index)
{
	if(file_index >= dramfs_num_files || !dramfs_valid)
		return (const void*)"";
	
	//Look up the contents pointer in the directory
	unsigned int *directory = (unsigned int*)(DRAMFS_LOC + 16);
	unsigned int offset = directory[(file_index*3)+1];
	
	return (const void*)(DRAMFS_LOC+offset);
}

size_t dramfs_getlen(unsigned int file_index)
{
	if(file_index >= dramfs_num_files || !dramfs_valid)
		return 0;
	
	//Look up the length in the directory
	unsigned int *directory = (unsigned int*)(DRAMFS_LOC + 16);
	return directory[(file_index*3)+2];
}

unsigned int dramfs_findfile(const char *filename)
{
	//Linear search for now
	for(unsigned int i = 0; i < dramfs_num_files; i++)
	{
		if(!strcmp(filename, dramfs_getname(i)))
			return i;
	}
	
	fprintf(stderr, "Warning: DRAMFS failed to find %s\n", filename);
	return -1;
}

unsigned int dramfs_numfiles()
{
	return dramfs_num_files;
}

unsigned int dramfs_isvalid()
{
	return dramfs_valid;
}