//solid16.c
//Decodes direct-color BMP and turns it into simpler RGBX5551 format for rendering
//Bryan Topp <betopp@betopp.com> 2016

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

int main(int argc, const char **argv)
{
	if(argc != 3)
	{
		printf("Usage: %s <bmp> <output filename>\n", argv[0]);
		exit(-1);
	}
	
	FILE *bmpfile = fopen(argv[1], "rb");
	assert(bmpfile);
	
	size_t bmpsize;
	fseek(bmpfile, 0, SEEK_END);
	bmpsize = ftell(bmpfile);
	fseek(bmpfile, 0, SEEK_SET);
	
	void *bmpdata = malloc(bmpsize);
	assert(bmpdata);
	size_t bytes_read = fread(bmpdata, 1, bmpsize, bmpfile);
	assert(bytes_read == bmpsize);
	
	FILE *pxfile = fopen(argv[2], "wb");
	assert(pxfile);
	
	printf("Pixels -> %s\n", argv[2]);

	unsigned int pixel_offset = *((unsigned int*)(bmpdata+0x0A));
	
	printf("Offset to pixel data is %u\n", pixel_offset);
	
	unsigned int bmp_width = *((unsigned int*)(bmpdata+0x12)); 
	unsigned int bmp_height = *((unsigned int*)(bmpdata+0x16)); 
	
	printf("Width is %u\n", bmp_width);
	printf("Height is %u\n", bmp_height);
	
	unsigned int bmp_colors = *((unsigned int*)(bmpdata+0x1C)); 
	assert(bmp_colors == 24); //Must be direct-color, no alpha
	
	unsigned int compression = *((unsigned int*)(bmpdata+0x1E));
	assert(compression == 0);
	
	unsigned char *bmp_pixels = (unsigned char*)(bmpdata) + pixel_offset;

	//write "header"
	const char *hdr = "TTP16BPP";
	fwrite(hdr, 1, 8, pxfile);
	fwrite(&bmp_width, 1, 4, pxfile);
	fwrite(&bmp_height, 1, 4, pxfile);


	short dither_matrix[4][4] = {
		{-4, 0, -3, 1},
		{2, -2, 3, -1},
		{-3, 1, -4, 0},
		{3, -1, 2, -2}
	};

	int row;
	for(row = bmp_height-1; row >= 0; row--)
	{
		int col;
		for(col = 0; col < bmp_width; col++)
		{
			//Yeah, we could just take X1R5G5B5 BMP directly, but I'd rather have control over the dither here.
			//(GIMP doesn't offer control over bit-depth right now, so its BMP exporter would do something arbitrary anyway.)
			
			short b8 = bmp_pixels[((row*bmp_width)+col)*3];
			short g8 = bmp_pixels[(((row*bmp_width)+col)*3)+1];
			short r8 = bmp_pixels[(((row*bmp_width)+col)*3)+2];
			
			b8 += dither_matrix[row%4][col%4];
			g8 += dither_matrix[row%4][col%4];
			r8 += dither_matrix[row%4][col%4];
			
			if(b8 < 0)
				b8 = 0;
			else if(b8 > 255)
				b8 = 255;
			
			if(g8 < 0)
				g8 = 0;
			else if(g8 > 255)
				g8 = 255;
			
			if(r8 < 0)
				r8 = 0;
			else if(r8 > 255)
				r8 = 255;
			
			
			unsigned short outcolor;
			
			outcolor = 0x8000; //This is going to turn into ~SYNC when we copy it
			outcolor |= ((b8/8) << 10) & (0x7C00);
			outcolor |= ((g8/8) << 5)  & (0x03E0);
			outcolor |= ((r8/8) << 0)  & (0x001F);
			
			fwrite(&outcolor, 1, 2, pxfile);
		}
	}

	free(bmpdata);
	
	fclose(pxfile);
	fclose(bmpfile);
	
	return 0;
}
