//unbuild_dramfs.c
//Explodes a DRAMFS blob into files
//Bryan Topp <betopp@betopp.com> 2015

#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <stdlib.h>

int main(int argc, const char **argv)
{
	if(argc != 3)
	{
		printf("usage: %s <dramfs blob> <output dir>\n", argv[0]);
		exit(-1);
	}
	
	//Open the blob and read it into memory
	FILE *blobfile = fopen(argv[1], "rb");
	assert(blobfile);
	
	
	fseek(blobfile, 0, SEEK_END);
	size_t blobsize = ftell(blobfile);
	fseek(blobfile, 0, SEEK_SET);
	
	printf("Allocating and reading %lu bytes.\n", blobsize);
	
	void *blobdata = malloc(blobsize);
	assert(blobdata);
	
	size_t bytes_read = fread(blobdata, 1, blobsize, blobfile);
	assert(bytes_read == blobsize);

	//Ditch the file handle
	fclose(blobfile);
	
	assert(!memcmp("betdramfs_00", blobdata, 12));
	
	printf("Magic valid, parsing...\n");
	
	unsigned int num_files = *((unsigned int*)(blobdata+12));
	printf("Reading %u files.\n", num_files);
	
	unsigned int file_index;
	for(file_index = 0; file_index < num_files; file_index++)
	{
		unsigned int location_of_filename = *((unsigned int*)(blobdata+16+(sizeof(unsigned int)*file_index*3)));
		unsigned int location_of_contents = *((unsigned int*)(blobdata+16+(sizeof(unsigned int)*((file_index*3)+1))));
		unsigned int length_of_file = *((unsigned int*)(blobdata+16+(sizeof(unsigned int)*((file_index*3)+2))));
		
		printf("File %u (%s): %u +%u\n", file_index, (char*)(blobdata + location_of_filename), location_of_contents, length_of_file);
	}
	
	printf("Actually dumping file contents is not supported yet.\n");


	return 0;
}