//exceptions.c
//Blackfin exception handlers for BF533 Game System
//Bryan Topp <betopp@betopp.com> 2016

//Only used on real hardware
#ifndef JSYS_PC_MOCKUP

#include <stdio.h>
#include <sys/platform.h>


//Exception handler
__attribute__((exception_handler)) void exception_handler_func()
{	
	//Get exception type
	register unsigned int seqstat asm ("SEQSTAT");
	register void *retx asm("RETX");
	unsigned short excause = seqstat & SEQSTAT_EXCAUSE;
	void *badaddr = retx;

	//Turn off DCPLBs so we don't double-fault here if they're bad
	*pDMEM_CONTROL &= ~ENDCPLB;
	
	fprintf(stderr, "EXCEPTION OCCURRED!\n");	
	fprintf(stderr, "EXCAUSE: 0x%.2x\n", excause);
	fprintf(stderr, "RETX:    %p\n", badaddr);
	fprintf(stderr, "\n");
	
	switch(excause)
	{
		case 0x26:
		{
			fprintf(stderr, "Data access CPLB miss.\n");
			fprintf(stderr, "DCPLB_FAULT_ADDR:   %p\n", *pDCPLB_FAULT_ADDR);
			break;
		}
		default:
		{
			fprintf(stderr, "Unhandled exception.\n");
			break;
		}
	}
	
	while(1) {}
}

//Initialization code to install exception handler.
void exception_init()
{
	printf("Registering exception handler...\n");
	
	//Set interrupt vector
	*pEVT3 = &exception_handler_func;
}

#else // Mockup code follows

void exception_init()
{

}

#endif