//fram.h
//FRAM interface - nonvolatile configuration/scores
//Bryan Topp <betopp@betopp.com> 2016

#pragma once

//Initializes and checks FRAM
void fram_init();

//Reads FRAM into shadow
void fram_read();

//Writes FRAM from shadow
void fram_write();

//Re-initializes contents of FRAM
void fram_nuke();

//Inserts high score
void fram_insert_highscore(unsigned int score, unsigned char *name, unsigned char maxlevel);

//FRAM layout as structures
typedef struct fram_highscore_s {
	unsigned int score;
	unsigned char name[4]; //Fourth should be NUL
	unsigned char maxlevel;
} fram_highscore_t;

#define FRAM_NUM_SCORES 10
typedef struct fram_s {
	//High score table
	fram_highscore_t highscores[FRAM_NUM_SCORES];
	
	//Bookkeeping
	unsigned int coins_deposited_1;
	unsigned int coins_deposited_2;
	
	//Checksum (updated automatically by fram_write)
	unsigned int checksum;
} fram_t;

//Shadowed FRAM
extern fram_t fram_shadow;
