//shooty_fleet.h
//Enemy fleet control for shooty game
//Bryan Topp <betopp@betopp.com> 2016

#pragma once

#include "shooty_common.h"

//Init fleet for new stage
void shooty_fleet_initstage(unsigned char level);

//Update/tick function for fleet
void shooty_fleet_update();

//Fills sprite table with fleet data
void shooty_fleet_fillsprites();

//Damages a fleet ship
void shooty_fleet_damage(unsigned char which, unsigned short amt, unsigned char causing_player);

//Other things need to know about fleet ships for collision
typedef struct fleetship_s {
	unsigned short type; //Which type, in the fleet type table, this ship belongs to
	short health; //How much health remains
	short pos_x; //Position, X, pixels
	short pos_y; //Position, Y, pixels
	unsigned short behavior_step; //What frame of the behavior we're on
	unsigned short desired_sprite; //What their "normal" sprite should be, set by behavior
} fleetship_t;

extern fleetship_t fleetships[FLEET_MAX];

//Sprites (precached pointers from DRAMFS)
enum FleetSpriteData
{
	FLEET_SPRITE_DATA_DEFAULT = 0,
	
	FLEET_SPRITE_DATA_CHARGER_SW,
	FLEET_SPRITE_DATA_CHARGER_SSW,
	FLEET_SPRITE_DATA_CHARGER_S,
	FLEET_SPRITE_DATA_CHARGER_SSE,
	FLEET_SPRITE_DATA_CHARGER_SE,
	
	FLEET_SPRITE_DATA_MEDUSA,
	
	FLEET_SPRITE_DATA_ARC,
	
	FLEET_SPRITE_DATA_UFO,
	
	FLEET_SPRITE_DATA_TANKER,
	
	FLEET_SPRITE_DATA_SWEEPER_W,
	FLEET_SPRITE_DATA_SWEEPER_S,
	FLEET_SPRITE_DATA_SWEEPER_E,
	
	FLEET_SPRITE_DATA_ROLLER,
	
	FLEET_SPRITE_DATA_MAX
};

static const char * const shooty_fleet_sprite_data_names[FLEET_SPRITE_DATA_MAX] = 
{
	"fleet/fleet_default.8bpp",
	
	"fleet/charger_sw.8bpp",
	"fleet/charger_ssw.8bpp",
	"fleet/charger_s.8bpp",
	"fleet/charger_sse.8bpp",
	"fleet/charger_se.8bpp",
	
	"fleet/medusa.8bpp",
	
	"fleet/arc.8bpp",
	
	"fleet/ufo.8bpp",
	
	"fleet/tanker.8bpp",
	
	"fleet/sweeper_w.8bpp",
	"fleet/sweeper_s.8bpp",
	"fleet/sweeper_e.8bpp",
	
	"fleet/roller.8bpp"
};

extern const unsigned char *shooty_fleet_sprite_datas[FLEET_SPRITE_DATA_MAX];

void shooty_fleet_behavior_invalid(fleetship_t *ship);
void shooty_fleet_behavior_charger(fleetship_t *ship);
void shooty_fleet_behavior_medusa(fleetship_t *ship);
void shooty_fleet_behavior_arc(fleetship_t *ship);
void shooty_fleet_behavior_ufo(fleetship_t *ship);
void shooty_fleet_behavior_tanker(fleetship_t *ship);
void shooty_fleet_behavior_sweeper(fleetship_t *ship);
void shooty_fleet_behavior_roller(fleetship_t *ship);
