//shooty_fx.c
//FX for shooty game
//Bryan Topp <betopp@betopp.com> 2016

#include <string.h>
#include "spr.h"
#include "dramfs.h"

#include "shooty_common.h"
#include "shooty_fx.h"

//Table of effects currently playing
typedef struct fxparticle_s {
	short pos_x_12_4;
	short pos_y_12_4;
	
	short vel_x_12_4;
	short vel_y_12_4;
	
	short decay_denom;
	
	const unsigned char *sprdata; //All fx are 16px wide - one sprite wide, multiple tall in atlas
	
	//Animation parameters
	unsigned char current_frame; //Which animation frame we should be on now
	unsigned char total_frames; //Total number of animation frames in atlas
	unsigned char time_on_frame; //How many raster frames we've been on the current animation frame
	unsigned char time_per_frame; //How many raster frames to show each animation frame
} fxparticle_t;
fxparticle_t fxparticles[FX_MAX];

//Init function at new stage
void shooty_fx_initstage(unsigned char stagenum)
{
	memset(fxparticles, 0, sizeof(fxparticles));
	
	for(unsigned char f = 0; f < FX_MAX; f++)
	{
		//Just set the height and move offscreen now - sprite data gets set when spawned
		spr_spritetable[SIDX_FX + f].h = 16;
		spr_spritetable[SIDX_FX + f].y = 250;
	}
}

//Spawner
void shooty_fx_spawn(short x, short y, unsigned char fxtype)
{
	//Find an open spot
	unsigned char fx;
	for(fx = 0; fx < FX_MAX; fx++)
	{
		if(fxparticles[fx].current_frame >= fxparticles[fx].total_frames)
			break;
	}
	
	if(fx >= FX_MAX) //No free slots, don't spawn
		return;
	
	fxparticle_t *fxp = &(fxparticles[fx]);
	
	fxp->pos_x_12_4 = x << 4;
	fxp->pos_y_12_4 = y << 4;
	
	fxp->current_frame = 0;
	fxp->time_on_frame = 0;
	
	if(fxtype == FXTYPE_DEATH)
	{
		fxp->sprdata = dramfs_getptr(dramfs_findfile("fx/death.8bpp"));
		
		fxp->vel_x_12_4 = (shooty_prng16() % 8);
		fxp->vel_y_12_4 = (shooty_prng16() % 8);
		
		fxp->total_frames = 6;
		fxp->time_per_frame = 4;
		
		fxp->decay_denom = 4;
	}
	else if(fxtype == FXTYPE_HIT)
	{
		fxp->sprdata = dramfs_getptr(dramfs_findfile("fx/hit.8bpp"));
		
		fxp->vel_x_12_4 = (shooty_prng16() % 16) - 8;
		fxp->vel_y_12_4 = (shooty_prng16() % 16) - 8;
		
		fxp->total_frames = 6;
		fxp->time_per_frame = 4;
		
		fxp->decay_denom = 4;
	}
	else if(fxtype >= FXTYPE_POINTS_000 && fxtype <= FXTYPE_POINTS_900)
	{
		fxp->sprdata = dramfs_getptr(dramfs_findfile("score/hundreds.8bpp")) + (16*16*(fxtype-FXTYPE_POINTS_000));
		
		fxp->vel_x_12_4 = -8;
		fxp->vel_y_12_4 = -16;
		
		fxp->total_frames = 1;
		fxp->time_per_frame = 40;
		
		fxp->decay_denom = 4;
	}
	else
	{
		fxp->total_frames = 0;
		fxp->time_per_frame = 0;
		fxp->decay_denom = 1;
	}
	
	
}

void shooty_fx_spawn_points_helper(short x, short y, unsigned short pointvalue)
{
	unsigned char pvfx = FXTYPE_POINTS_000 + (pointvalue / 100);
	shooty_fx_spawn(x,y,pvfx);
}

//Updater
void shooty_fx_update()
{
	//Spawn floaters sometimes
	/*static unsigned char floater_tickdown;
	static unsigned short floater_pos;
	if(!(floater_tickdown-- % 32))
	{
		shooty_fx_spawn( (floater_pos % 256) + 32, -16, FXTYPE_FLOATER);
		floater_pos = (floater_pos * 5113) + 3469;
	}*/
	


	for(unsigned char f = 0; f < FX_MAX; f++)
	{
		if(fxparticles[f].current_frame >= fxparticles[f].total_frames)
			continue;
		
		fxparticles[f].time_on_frame++;
		if(fxparticles[f].time_on_frame >= fxparticles[f].time_per_frame)
		{
			fxparticles[f].current_frame++;
			fxparticles[f].time_on_frame = 0;
		}
		
		fxparticles[f].pos_x_12_4 += fxparticles[f].vel_x_12_4;
		fxparticles[f].pos_y_12_4 += fxparticles[f].vel_y_12_4;
		
		fxparticles[f].vel_x_12_4 -= fxparticles[f].vel_x_12_4 / fxparticles[f].decay_denom;
		fxparticles[f].vel_y_12_4 -= fxparticles[f].vel_y_12_4 / fxparticles[f].decay_denom;

	}
}

//Sprite filler
void shooty_fx_fillsprites()
{
	//Set sprites for FX
	for(unsigned char f = 0; f < FX_MAX; f++)
	{
		if(fxparticles[f].current_frame < fxparticles[f].total_frames)
		{
			spr_spritetable[SIDX_FX+f].x = fxparticles[f].pos_x_12_4 >> 4;
			spr_spritetable[SIDX_FX+f].y = fxparticles[f].pos_y_12_4 >> 4;
			spr_spritetable[SIDX_FX+f].data = fxparticles[f].sprdata + (256*fxparticles[f].current_frame);
		}
		else
		{
			spr_spritetable[SIDX_FX+f].y = 250;
		}
	}
}