//spiutils.h
//Helpers for the DRAM loading, FRAM manipulation, and IOCON polling
#pragma once
#ifndef JSYS_PC_MOCKUP

#include <sys/platform.h>

//SPI slave select pin locations
#define SPI_SLAVE_ROM_ENABLE FLS2 //SPI_ROMSEL = PF2
#define SPI_SLAVE_FRAM_ENABLE FLS1 //SPI_FRAMSEL = PF1
#define SPI_SLAVE_IOCON_ENABLE FLS3 //SPI_IOSEL = PF3
#define SPI_SLAVE_ENABLES (SPI_SLAVE_ROM_ENABLE | SPI_SLAVE_FRAM_ENABLE | SPI_SLAVE_IOCON_ENABLE)

//Active-low chip selects
#define SPI_SLAVE_ROM_SELECT ((0xFF00 & ~FLG2) | SPI_SLAVE_ENABLES)
#define SPI_SLAVE_FRAM_SELECT ((0xFF00 & ~FLG1) | SPI_SLAVE_ENABLES)
#define SPI_SLAVE_IOCON_SELECT ((0xFF00 & ~FLG3) | SPI_SLAVE_ENABLES)
#define SPI_SLAVE_NONE_SELECT (0xFF00 | SPI_SLAVE_ENABLES)

//IORST = PF0


static inline void spi_set_iocon()
{
	//Set slave-select pins active and deselect all
	*pSPI_FLG = SPI_SLAVE_NONE_SELECT;

	//System clock is 98.304MHz
	//Set SPI_BAUD to 256 -> division by 512 -> 192KHz SPI baud
	//According to the datasheet, baud rate under ~2MHz necessary for IOCON (8MHz ATTiny clock / 4)
	//Seems to break horribly when we go above 200KHz...???
	*pSPI_BAUD = 256;

	//SPI Master mode
	//Enable SPI
	//SPI clock phase 1
	//SPI clock polarity 1
	//Transmit zeroes when transmit buffer empties
	//Initiate on transmit buffer write
	*pSPI_CTL = MSTR | SPE | CPHA | CPOL | SZ | TDBR_CORE;
	
	//Select IOCON	
	*pSPI_FLG = SPI_SLAVE_IOCON_SELECT;
}

static inline void spi_set_rom()
{
	//Set slave-select pins active and deselect all
	*pSPI_FLG = SPI_SLAVE_NONE_SELECT;

	//System clock is 98.304MHz
	//Set SPI_BAUD to 2 -> division by 4 -> 24.576MHz SPI baud
	//Should take about 5 seconds to read the whole thing at that rate.
	*pSPI_BAUD = 2;

	//SPI Master mode
	//Enable SPI
	//SPI clock phase 1
	//SPI clock polarity 1
	//Transmit zeroes when transmit buffer empties
	//Initiate on transmit buffer write
	*pSPI_CTL = MSTR | SPE | CPHA | CPOL | SZ | TDBR_CORE;
	
	//Select Flash ROM
	*pSPI_FLG = SPI_SLAVE_ROM_SELECT;
}

static inline void spi_set_fram()
{
	//Set slave-select pins active and deselect all
	*pSPI_FLG = SPI_SLAVE_NONE_SELECT;

	//System clock is 98.304MHz
	//Set SPI_BAUD to 4 -> division by 8 -> 12.288MHz SPI baud
	//FRAM goes up to 20MHz, so this should be fine
	*pSPI_BAUD = 4;

	//SPI Master mode
	//Enable SPI
	//SPI clock phase 1
	//SPI clock polarity 1
	//Transmit zeroes when transmit buffer empties
	//Initiate on transmit buffer write
	*pSPI_CTL = MSTR | SPE | CPHA | CPOL | SZ | TDBR_CORE;
	
	//Select FRAM
	*pSPI_FLG = SPI_SLAVE_FRAM_SELECT;
}

static inline void spi_set_none()
{
	//Just leave the old settings and deselect the device
	*pSPI_FLG = SPI_SLAVE_NONE_SELECT;
}

static inline unsigned char spi_exchange(unsigned char tx)
{	
	*pSPI_STAT &= ~RXS;
	*pSPI_TDBR = tx;
	
	//Wait
	ssync();
	while(!(*pSPI_STAT & RXS)) {}
	ssync();
		
	//Get what came back
	return *pSPI_RDBR;
}

#endif // !JSYS_PC_MOCKUP