//spr.h
//Actual sprite/plane rendering methods
//Called by the video system in hblank interrupts
//Input data should come from application program
//Bryan Topp <betopp@betopp.com> 2015
//Inspired by the Neo-Geo LSPC design

#pragma once

#define SPR_TEXTPLANE_COLS 64
#define SPR_TEXTPLANE_ROWS 32

#define SPR_ACTIVE_PX 320

//----------------------------------------
//Main interface
//----------------------------------------

//Initializes rendering engine
void spr_init();

//Called from DMA line-end interrupt (or from SDL2 rendering thread in mockup)
void spr_render_line(unsigned short *buffer_out, unsigned char y);


//----------------------------------------
//Test patterns
//----------------------------------------

//Set this to nonzero to enable VDAC test patterns at line-render time
#define SPR_TESTPAT_OFF 0
#define SPR_TESTPAT_DOT 1
#define SPR_TESTPAT_CROSS 2
#define SPR_TESTPAT_BARS 3
extern unsigned short spr_testpattern;


//----------------------------------------
//Text plane (top)
//----------------------------------------

extern char spr_textplane[SPR_TEXTPLANE_ROWS][SPR_TEXTPLANE_COLS]; //2D plane of 7-bit ASCII characters
extern unsigned short spr_textplane_color;

//Clears the text plane and resets its colors
void spr_textplane_clear();

//----------------------------------------
//Sprites (under text plane, above background)
//----------------------------------------

typedef struct spr_spritedef_s
{
	//Pointer to pixel data (16px wide, 8bpp)
	const unsigned char *data;
	
	//X position of the top-left corner.
	short x;
	//Y position of the top-left corner.
	short y;
	
	//Height of sprite
	unsigned short h;
	
} spr_spritedef;

extern spr_spritedef spr_spritetable[];
extern unsigned char spr_sprite_enable;

//Clears all sprites (0 height)
void spr_sprite_clear();

//Sprite chaining token - use this as X or Y coordinate to chain sprites
#define SPR_CHAIN_COORDINATE (-1024)

//----------------------------------------
//Palette for sprites
//----------------------------------------

//Takes an array of BGRZ pixel data, flattens to RGB555, and loads sprite palette with it
void spr_loadpalette_bgr(const unsigned int *bgrz8888);

//----------------------------------------
//Background layer (under everything)
//----------------------------------------

//Points to background graphics plane file
//Should be 8 bytes of dummy (magic), 4-byte width, 4-byte height, then 16-bit pixels data
extern const unsigned short *spr_background;
extern short spr_background_scroll_v;
